import typing
import unittest

import jsonpickle
import selenium
import tzlocal

from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.by import By

from tauk.context.context import TaukContext
from tauk.context.test_case import TestCase
from tauk.context.test_data import TestData
from tauk.context.test_suite import TestSuite
from tauk.enums import TestStatus, AutomationTypes, PlatformNames, BrowserNames
from tauk.listeners.unittest_listener import TaukListener
from tauk.tauk_webdriver import Tauk
from tests.utils import mock_success


class TestDataTest(unittest.TestCase):
    expected_run_id = '6d917db6-cf5d-4f30-8303-6eefc35e7558'
    options = Options()
    driver = None

    @classmethod
    def setUpClass(cls) -> None:
        cls.options = Options()
        cls.options.headless = True
        cls.driver = webdriver.Chrome(options=cls.options)

    @classmethod
    def tearDownClass(cls) -> None:
        if hasattr(cls, 'driver') and cls.driver:
            cls.driver.quit()

    def setUp(self) -> None:
        pass

    def tearDown(self) -> None:
        pass

    @mock_success()
    def test_1_success_test_case_data(self):
        Tauk.register_driver(TestDataTest.driver)
        self.driver.get('https://www.tauk.com')

        def validate(file_name, method_name, ctx: TaukContext, test_data: TestData, test_suite: TestSuite,
                     test_case: TestCase):
            json_data = jsonpickle.decode(ctx.get_json_test_data(file_name, method_name))
            tc = json_data['test_suites'][0]['test_cases'][0]
            self.assertEqual(tc['custom_name'], 'custom name', 'Invalid custom name')
            self.assertEqual(tc['method_name'], 'test_success_test_case_data', 'Invalid method name')
            self.assertEqual(tc['status'], TestStatus.PASSED.value, 'Invalid test status')
            self.assertEqual(tc['automation_type'], AutomationTypes.SELENIUM.value, f'Invalid automation type')
            self.assertEqual(tc['platform_name'], PlatformNames.MACOS.value, 'Invalid platform name')
            self.assertEqual(tc['browser_name'], BrowserNames.CHROME.value, 'Invalid browser name')
            self.assertTrue(len(tc['browser_version']) > 0, f'Invalid browser version {tc["browser_version"]}')
            self.assertEqual(len(str(tc['start_timestamp'])), 13, f'Invalid start timestamp {tc["start_timestamp"]}')
            self.assertEqual(len(str(tc['end_timestamp'])), 13, f'Invalid end timestamp {tc["end_timestamp"]}')
            self.assertTrue(tc['end_timestamp'] > tc['start_timestamp'],
                            f'Invalid start[{tc["start_timestamp"]}] and end timestamp[{tc["end_timestamp"]}]')
            self.assertEqual(tc['timezone'], tzlocal.get_localzone_name(), 'Invalid timezone')
            self.assertTrue(len(tc['screenshot']) > 100, 'Invalid screenshot')
            self.assertTrue(len(tc['view']) > 100, 'Invalid view')
            self.assertEqual(tc['webdriver_client_version'], selenium.__version__, 'Invalid webdriver client version')
            self.assertEqual(tc['browser_driver_version'], self.driver.capabilities['chrome']['chromedriverVersion'],
                             'Invalid chrome driver version')
            self.assertDictEqual(tc['capabilities'], self.driver.capabilities, 'capabilities does not match')

            self.assertNotIn('error', tc.keys(), 'error is not None')
            self.assertNotIn('code_context', tc.keys(), 'code_context is not None')
            self.assertNotIn('excluded', tc.keys(), 'excluded is not None')
            self.assertNotIn('appium_server_version', tc.keys(), 'appium_server_version is not None')
            self.assertNotIn('tags', tc.keys(), 'tags is not None')
            self.assertNotIn('user_data', tc.keys(), 'user_data is not None')
            self.assertNotIn('log', tc.keys(), 'log is not None')

        return validate

    def validate_failure(file_name, method_name, ctx: TaukContext, test_data: TestData, test_suite: TestSuite,
                         test_case: TestCase):
        json_data = jsonpickle.decode(ctx.get_json_test_data(file_name, method_name))
        tc = json_data['test_suites'][0]['test_cases'][0]
        t = unittest.TestCase()
        t.assertEqual(tc['method_name'], 'test_2_failure_test_case_data', 'Invalid method name')
        t.assertEqual(tc['status'], TestStatus.FAILED.value, 'Invalid test status')
        t.assertEqual(tc['aucatomation_type'], AutomationTypes.SELENIUM.value, f'Invalid automation type')
        t.assertEqual(tc['platform_name'], PlatformNames.MACOS.value, 'Invalid platform name')
        t.assertEqual(tc['browser_name'], BrowserNames.CHROME.value, 'Invalid browser name')
        t.assertTrue(len(tc['browser_version']) > 0, f'Invalid browser version {tc["browser_version"]}')
        t.assertEqual(len(str(tc['start_timestamp'])), 13, f'Invalid start timestamp {tc["start_timestamp"]}')
        t.assertEqual(len(str(tc['end_timestamp'])), 13, f'Invalid end timestamp {tc["end_timestamp"]}')
        t.assertTrue(tc['end_timestamp'] > tc['start_timestamp'],
                     f'Invalid start[{tc["start_timestamp"]}] and end timestamp[{tc["end_timestamp"]}]')
        t.assertEqual(tc['timezone'], tzlocal.get_localzone_name(), 'Invalid timezone')
        t.assertTrue(len(tc['screenshot']) > 100, 'Invalid screenshot')
        t.assertTrue(len(tc['view']) > 100, 'Invalid view')
        t.assertEqual(tc['webdriver_client_version'], selenium.__version__, 'Invalid webdriver client version')
        t.assertEqual(tc['browser_driver_version'], TestDataTest.driver.capabilities['chrome']['chromedriverVersion'],
                      'Invalid chrome driver version')
        t.assertDictEqual(tc['capabilities'], TestDataTest.driver.capabilities, 'Capabilitites does not match')
        t.assertIsInstance(tc['code_context'], typing.List, 'code_context must be of list type')
        t.assertTrue(len(tc['code_context']) > 0, 'code_context should not be a empty list')
        t.assertCountEqual(tc['error'].keys(),
                           ['error_type', 'error_msg', 'line_number', 'invoked_func', 'code_executed'])
        t.assertEqual(tc['error']['error_type'], 'NoSuchElementException')
        t.assertTrue(len(tc['error']['error_msg']) > 0)
        t.assertTrue(tc['error']['line_number'] > 0)
        t.assertEqual(tc['error']['invoked_func'], 'test_2_failure_test_case_data')
        t.assertTrue(len(tc['error']['code_executed']) > 0)

        t.assertNotIn('custom_name', tc.keys(), 'custom_name is not None')
        t.assertNotIn('excluded', tc.keys(), 'excluded is not None')
        t.assertNotIn('appium_server_version', tc.keys(), 'appium_server_version is not None')
        t.assertNotIn('tags', tc.keys(), 'tags is not None')
        t.assertNotIn('user_data', tc.keys(), 'user_data is not None')
        t.assertNotIn('log', tc.keys(), 'log is not None')

    @unittest.expectedFailure
    @mock_success(validation=validate_failure)
    def test_2_failure_test_case_data(self):
        Tauk.register_driver(self.driver)
        self.driver.get('https://www.tauk.com')
        self.driver.find_element(By.ID, 'unknown-id')


if __name__ == '__main__':
    Tauk(api_token="api-token", project_id="project-id")
    suite = unittest.defaultTestLoader.loadTestsFromTestCase(TestDataTest)
    unittest.TextTestRunner(resultclass=TaukListener).run(suite)



import logging
import re
import requests

logger = logging.getLogger('tauk')


def get_appium_server_version(driver):
    driver_url = driver.command_executor._url
    response = requests.get(f'{driver_url}/status')
    if response.status_code == 200:
        try:
            json_response = response.json()
            return json_response['value']['build']['version']
        except KeyError:
            pass
    return None


def get_browser_driver_version(driver):
    browser_name = driver.capabilities.get('browserName', '')
    if browser_name == 'chrome':
        return driver.capabilities['chrome']['chromedriverVersion']
    elif browser_name == 'firefox':
        return driver.capabilities['moz:geckodriverVersion']
    elif browser_name == 'msedge':
        return driver.capabilities['msedge']['msedgedriverVersion']
    else:
        return None


def shortened_json(json_text):
    json_text = re.sub(r'"screenshot": ".+?"', '"screenshot" : "stripped"', json_text, flags=re.DOTALL)
    json_text = re.sub(r'"view": ".+?", ', '"view" : "stripped", ', json_text, flags=re.DOTALL)
    json_text = re.sub(r'"log": \[.+?}\], "', '"log": ["stripped"], "', json_text, flags=re.DOTALL)
    return json_text



import time
import unittest

from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.by import By

from tauk.tauk_webdriver import Tauk
from tests.utils import mock_success


class UnitTestListenerTestOne(unittest.TestCase):

    def tearDown(self) -> None:
        self.driver.quit()
        pass

    @mock_success(multiprocess=True)
    def test_one(self):
        print('running test one')
        options = Options()
        # options.headless = True

        self.driver = webdriver.Chrome(options=options)
        Tauk.register_driver(self.driver)
        self.driver.get('https://www.tauk.com')
        # self.driver.find_element(By.ID, 'nonexisting')



import logging
import re
import subprocess

import responses

from tauk.tauk_webdriver import Tauk

logger = logging.getLogger('tauk')

if __name__ == '__main__':
    expected_run_id = '6d917db6-cf5d-4f30-8303-6eefc35e7558'
    responses.RequestsMock()
    responses.start()
    responses.add(responses.POST, re.compile(r'https://www.tauk.com/api/v1/execution/.+/initialize'),
                  json={'run_id': expected_run_id, 'latest_tauk_client_version': '2.0.1'}, status=200)

    try:
        logger.info('Initializing main tauk instance')
        Tauk(api_token='api-token', project_id='project-id', multi_process_run=True)

        p1 = subprocess.Popen(['python', 'tests/e2e/listeners/multiprocess/launcher.py',
                               '--test-name=test_one', '--class-name=UnitTestListenerTestOne']
                              )
        p2 = subprocess.Popen(['python', 'tests/e2e/listeners/multiprocess/launcher.py',
                               '--test-name=test_two', '--class-name=UnitTestListenerTestTwo']
                              )
        p1_out, p1_err = p1.communicate()
        print('returncode:', p1.returncode)
        print('out:', p1_out)
        print('err:', p1_err)

        p2_out, p2_err = p2.communicate()
        print('returncode:', p2.returncode)
        print('out:', p2_out)
        print('err:', p2_err)
    finally:
        responses.stop()



import inspect
import logging
import os
import re
import typing
from functools import wraps

import responses

from tauk.exceptions import TaukException
from tauk.tauk_webdriver import Tauk

logger = logging.getLogger('tauk')


def enable_mocking():
    responses.RequestsMock()
    responses.start()
    logger.debug('Enabled mocking')


def disable_mocking():
    responses.stop()
    logger.debug('Disabled mocking')


def register_mock_urls(urls: typing.List[str], json_responses: typing.List[object], statuses: typing.List[int]):
    for i, url in enumerate(urls):
        logger.debug(f'Registering mock url: {url}')
        responses.add(responses.POST, url, json=json_responses[i], status=statuses[i])


def mock(urls: [], json_responses: typing.List[object], statuses: typing.List[int], validation=None,
         multiprocess=False, init_tauk=False):
    if not isinstance(urls, list) or not isinstance(json_responses, list) or not isinstance(statuses, list):
        raise TaukException('arguments to mock method must be of list type of same size')

    if not (len(urls) == len(json_responses) == len(statuses)):
        raise TaukException('arguments to mock method must be of list type of same size')

    enable_mocking()
    register_mock_urls(urls, json_responses, statuses)

    if init_tauk and not Tauk.is_initialized():
        api_token = os.getenv('TAUK_API_TOKEN', 'api-token')
        project_id = os.getenv('TAUK_PROJECT_ID', 'project-id')
        Tauk(api_token=api_token, project_id=project_id, multi_process_run=multiprocess)

    def inner_decorator(func):
        caller_filename = None
        test_method_name = func.__name__
        for frame_info in inspect.stack():
            if all(elem in frame_info.frame.f_code.co_names for elem in ['Tauk', 'observe']):
                caller_filename = frame_info.filename.replace(f'{os.getcwd()}{os.sep}', '')
                break

        @wraps(func)
        def inner(*args, **kwargs):
            try:
                result = func(*args, **kwargs)

                if callable(result):
                    ctx = Tauk().get_context()
                    test_data = ctx.test_data
                    test_suite = test_data.get_test_suite(caller_filename)
                    if test_suite:
                        test_case = test_suite.get_test_case(test_method_name)
                        if test_case:
                            result(caller_filename, test_method_name, ctx, test_data, test_suite, test_case)

                return result
            except:
                if validation and callable(validation):
                    ctx = Tauk().get_context()
                    test_data = ctx.test_data
                    test_suite = test_data.get_test_suite(caller_filename)
                    if test_suite:
                        test_case = test_suite.get_test_case(test_method_name)
                        if test_case:
                            validation(caller_filename, test_method_name, ctx, test_data, test_suite, test_case)

                raise
            finally:
                # disable_mocking()
                pass

        return inner

    return inner_decorator


def mock_success(expected_run_id='6d917db6-cf5d-4f30-8303-6eefc35e7558', validation=None,
                 multiprocess=False, init_tauk=False):
    return mock(
        urls=[re.compile(r'https://www.tauk.com/api/v1/execution/.+/initialize'),
              re.compile(r'https://www.tauk.com/api/v1/execution/.+/.+/report/upload')],
        json_responses=[{'run_id': expected_run_id, 'message': 'success'},
                        {'message': 'success'}],
        statuses=[200, 200],
        validation=validation,
        multiprocess=multiprocess,
        init_tauk=init_tauk
    )



"""Helper package to facilitate reporting for webdriver-based tests on Tauk"""
import atexit
import inspect
import logging
import os
import sys
import unittest
from datetime import datetime, timezone
from functools import wraps
from threading import Lock

from tauk.context.context import TaukContext
from tauk.exceptions import TaukException
from tauk.context.test_data import TestCase

logger = logging.getLogger('tauk')

mutex = Lock()


class Tauk:
    instance = None
    __context: TaukContext

    # TODO: Add documentation
    def __new__(cls, api_token=None, project_id=None, multi_process_run=False, cleanup_exec_context=True):
        with mutex:
            if Tauk.instance is None:
                logger.debug(f'Creating new Tauk instance with api_token={api_token}, project_id={project_id}, '
                             f'multi_process_run={multi_process_run}')
                cls.instance = super(Tauk, cls).__new__(cls)
                if not api_token or not project_id:
                    api_token = os.getenv('TAUK_API_TOKEN')
                    project_id = os.getenv('TAUK_PROJECT_ID')
                    multi_process_run = os.getenv('TAUK_MULTI_PROCESS',
                                                  f'{multi_process_run}').lower().strip() == "true"
                    logger.info(f'Environment variables contains api_token={api_token}, project_id={project_id},'
                                f' multi_process_run={multi_process_run}')

                if not multi_process_run and (not api_token or not project_id):
                    raise TaukException('Please ensure that a valid TAUK_API_TOKEN and TAUK_PROJECT_ID is set')

                Tauk.__context = TaukContext(api_token, project_id, multi_process_run=multi_process_run)

                if multi_process_run and cleanup_exec_context:
                    atexit.register(Tauk.destroy)

            return cls.instance

    @classmethod
    def is_initialized(cls):
        return False if Tauk.instance is None else True

    @classmethod
    def get_instance(cls):
        if Tauk.instance is None:
            raise Exception('Tauk is not yet initialized')
        logger.info(f'Returning Tauk instance {cls.instance}')
        return cls.instance

    @classmethod
    def get_context(cls):
        return Tauk.__context

    @classmethod
    def _get_testcase(cls, file_name, test_name):
        test_suite = Tauk.__context.test_data.get_test_suite(file_name)
        if test_suite is None:
            return None
        test_case = test_suite.get_test_case(test_name)
        if test_case is None:
            return None
        return test_case

    @classmethod
    def destroy(cls):
        if Tauk.is_initialized():
            logger.debug('Destroying Tauk context')
            try:
                Tauk.__context.delete_execution_files()
            except Exception as ex:
                logger.error('Failed to delete execution file', exc_info=ex)

            del cls.instance

    # TODO: Move identifier to common method
    @classmethod
    def register_driver(cls, driver, unittestcase=None):
        if hasattr(unittestcase, 'tauk_skip') and unittestcase.tauk_skip is True:
            logger.info(f'register_driver: Skipping driver registration for [{unittestcase.id()}] ---')
            return

        logger.info(f'Registering driver instance: driver=[{driver}], unittestcase=[{unittestcase}]')
        if not Tauk.is_initialized():
            raise TaukException('driver can only be registered from test methods')

        if unittestcase:
            if not isinstance(unittestcase, unittest.TestCase):
                raise TaukException(
                    f'argument unittestcase ({type(unittestcase)}) is not an instance of unittest.TestCase')

            test_filename = inspect.getfile(unittestcase.__class__).replace(f'{os.getcwd()}{os.sep}', '')
            test_method_name = unittestcase.id().split('.')[-1]
            test = Tauk._get_testcase(test_filename, test_method_name)
            if test is None:
                raise TaukException(f'TaukListener was not attached to unittest runner')
            test.register_driver(driver)
            return

        caller_frame_records = inspect.stack()
        register_driver_stack_index = 0
        found_register_driver = False

        for i, frame_info in enumerate(caller_frame_records):
            # We pick the next frame after register driver
            if found_register_driver:
                test_filename = frame_info.filename
                test_relative_file_name = test_filename.replace(f'{os.getcwd()}{os.sep}', '')
                test_method_name = frame_info.function
                test = Tauk._get_testcase(test_relative_file_name, test_method_name)
                if test is None:
                    raise TaukException(
                        f'driver can only be registered with an active tauk listener or an observed method')
                test.register_driver(driver, test_filename, test_method_name)
                return

            if frame_info.function == 'register_driver':
                found_register_driver = True
                register_driver_stack_index = i

        raise TaukException(f'driver(2) can only be registered with an active tauk listener or an observed method')

    @classmethod
    def observe(cls, custom_test_name=None, excluded=False):

        def inner_decorator(func):
            logger.debug(f'Registering test method=[{func.__name__}]'
                         f' with custom_test_name=[{custom_test_name}], excluded=[{excluded}]')
            test_case = TestCase()
            test_case.custom_name = custom_test_name
            test_case.excluded = excluded

            all_frames = inspect.stack()
            caller_filename = None
            caller_relative_filename = None
            for frame_info in all_frames:
                if func.__name__ in frame_info.frame.f_code.co_names:
                    caller_filename = frame_info.filename
                    caller_relative_filename = caller_filename.replace(f'{os.getcwd()}{os.sep}', '')
                    test_case.method_name = func.__name__
                    Tauk() if not Tauk.is_initialized() else None
                    Tauk.__context.test_data.add_test_case(caller_relative_filename, test_case)
                    break

            @wraps(func)
            def invoke_test_case(*args, **kwargs):
                try:
                    test_case.start_timestamp = int(datetime.now(tz=timezone.utc).timestamp() * 1000)
                    result = func(*args, **kwargs)
                    test_case.end_timestamp = int(datetime.now(tz=timezone.utc).timestamp() * 1000)
                    test_case.capture_success_data()
                except:
                    test_case.end_timestamp = int(datetime.now(tz=timezone.utc).timestamp() * 1000)
                    test_case.capture_failure_data()
                    error_line_number = test_case.capture_error(caller_filename, sys.exc_info())
                    test_case.capture_test_steps(testcase=func, error_line_number=error_line_number)

                    # TODO: cleanup stack track here
                    raise
                else:
                    return result
                finally:
                    test_case.capture_appium_logs()
                    # TODO: Investigate about overloaded test name
                    json_test_data = Tauk.__context.get_json_test_data(caller_relative_filename, test_case.method_name)
                    Tauk.__context.api.upload(json_test_data)
                    Tauk.__context.test_data.delete_test_case(caller_relative_filename, test_case.method_name)

            return invoke_test_case

        return inner_decorator

    @classmethod
    def sync_data(cls):
        pass

    @classmethod
    def add_user_data(cls, name, value, test_file_name=None, test_method_name=None):
        if test_file_name and test_method_name:
            test = Tauk._get_testcase(test_file_name, test_method_name)
            if test is None:
                raise TaukException(f'user data can only be added withing the test method'
                                    f' Verify if {test_file_name} has @Tauk.observe decorator')
            test.add_user_data(name, value)
            return

        caller_frame_records = inspect.stack()
        found_register_driver = False

        for i, frame_info in enumerate(caller_frame_records):
            # We pick the next frame after register driver
            logger.info(f'[{i}] Checking function: {frame_info.function}')
            if found_register_driver:
                test_file_name = frame_info.filename.replace(os.getcwd(), '')
                test_method_name = frame_info.function
                logger.info(f'[{i}] Found: {test_file_name}, {test_method_name}')
                test = Tauk._get_testcase(test_file_name, test_method_name)
                if test is None:
                    raise TaukException(f'Driver can only be registered for observed methods.'
                                        f' Verify if {test_file_name} has @Tauk.observe decorator')
                test.add_user_data(name, value)
                return

            if frame_info.function == 'register_driver':
                found_register_driver = True
                register_driver_stack_index = i

        raise TaukException(f'Driver can only be registered for observed methods.'
                            f' Verify if {test_file_name} has @Tauk.observe decorator')



import inspect
import logging
import os
import traceback
import unittest
from datetime import datetime, timezone
from typing import Dict

from tauk.context.test_case import TestCase
from tauk.enums import TestStatus
from tauk.tauk_webdriver import Tauk

logger = logging.getLogger('tauk')


class TaukListener(unittest.TestResult):
    multiprocess_run = False

    def __init__(self, stream, descriptions, verbosity):
        self.tests = None
        self.test_filename = None
        super().__init__(stream, descriptions, verbosity)

    def _should_observe(self, test):
        return hasattr(test, 'tauk_skip') and test.tauk_skip is True

    def startTestRun(self) -> None:
        logger.info("# Test Run Started ---")
        self.tests: Dict[str, TestCase] = {}
        Tauk(multi_process_run=self.multiprocess_run) if not Tauk.is_initialized() else None
        super().startTestRun()

    def stopTestRun(self) -> None:
        logger.info('# Test Run Stopped ---')
        super().stopTestRun()

    def startTest(self, test: unittest.case.TestCase) -> None:
        if self._should_observe(test):
            logger.info(f'startTest: Skipping Tauk observe for the test [{test.id()}] ---')
            super().startTest(test)
            return

        logger.info(f'# Test Started [{test.id()}] ---')
        caller_filename = inspect.getfile(test.__class__)
        self.test_filename = caller_filename.replace(f'{os.getcwd()}{os.sep}', '')
        test_method_name = test.id().split('.')[-1]

        self.tests[test.id()] = TestCase()
        self.tests[test.id()].method_name = test_method_name
        self.tests[test.id()].start_timestamp = int(datetime.now(tz=timezone.utc).timestamp() * 1000)
        self.tests[test.id()].custom_name = test.shortDescription()

        Tauk.get_context().test_data.add_test_case(self.test_filename, self.tests[test.id()])

        super().startTest(test)

    def stopTest(self, test: unittest.case.TestCase) -> None:
        super().stopTest(test)
        if self._should_observe(test):
            logger.debug(f'stopTest: Skipping Tauk observe for the test [{test.id()}] ---')
            return

        logger.info(f'# Test Stopped [{test.id()}] ---')
        try:
            self.tests[test.id()].end_timestamp = int(datetime.now(tz=timezone.utc).timestamp() * 1000)
            self.tests[test.id()].capture_appium_logs()
            ctx = Tauk.get_context()

            caller_filename = inspect.getfile(test.__class__)
            caller_relative_filename = caller_filename.replace(f'{os.getcwd()}{os.sep}', '')
            ctx.api.upload(ctx.get_json_test_data(caller_relative_filename, self.tests[test.id()].method_name))
        except Exception as ex:
            logger.error('Failed to upload test results', exc_info=ex)

    def addError(self, test: unittest.case.TestCase, err: tuple) -> None:
        super().addError(test, err)
        if self._should_observe(test):
            logger.debug(f'addError: Skipping Tauk observe for the test [{test.id()}] ---')

            return

        logger.info(f'# Test Errored [{test.id()}] ---')
        self.tests[test.id()].status = TestStatus.FAILED

        exctype, value, tb = err
        traceback.print_exception(exctype, value, tb)
        self.tests[test.id()].capture_error(self.test_filename, err)

    def addFailure(self, test: unittest.case.TestCase, err: tuple) -> None:
        super().addFailure(test, err)
        if self._should_observe(test):
            logger.debug(f'addFailure: Skipping Tauk observe for the test [{test.id()}] ---')
            return

        logger.info(f'# Test Failed [{test.id()}] ---')
        self.tests[test.id()].status = TestStatus.FAILED

        exctype, value, tb = err
        traceback.print_exception(exctype, value, tb)
        self.tests[test.id()].capture_error(self.test_filename, err)

    def addSuccess(self, test: unittest.case.TestCase) -> None:
        super().addSuccess(test)
        if self._should_observe(test):
            logger.debug(f'addSuccess: Skipping Tauk observe for the test [{test.id()}] ---')
            return

        logger.info(f'# Test Passed [{test.id()}] ---')
        self.tests[test.id()].status = TestStatus.PASSED

    def addSkip(self, test: unittest.case.TestCase, reason: str) -> None:
        super().addSkip(test, reason)
        if self._should_observe(test):
            logger.debug(f'addSkip: Skipping Tauk observe for the test [{test.id()}] ---')
            return

        logger.info(f'# Test Skipped [{test.id()}] ---')
        self.tests[test.id()].excluded = True



import logging
import time
import typing
from datetime import datetime, timezone

import tzlocal

import tauk
from tauk.context.test_case import TestCase
from tauk.context.test_suite import TestSuite

logger = logging.getLogger('tauk')


class TestData:
    def __init__(self) -> None:
        self.tauk_client_version = tauk.__version__
        self.language = 'python'
        self.start_timestamp = int(datetime.now(tz=timezone.utc).timestamp() * 1000)
        self.timezone = tzlocal.get_localzone_name()
        self.dst = (time.localtime().tm_isdst != 0)
        self._test_suites: typing.List[TestSuite] = []

    @property
    def test_suites(self):
        return self._test_suites

    def get_test_suite(self, filename) -> TestSuite:
        for suite in self._test_suites:
            if suite.filename == filename:
                return suite
        return None

    def add_test_case(self, filename: str, test_case: TestCase):
        suite = self.get_test_suite(filename)
        if suite is None:
            suite = TestSuite(filename)
            self._test_suites.append(suite)

        suite.add_testcase(test_case)

    def delete_test_case(self, filename, test_method_name):
        suite = self.get_test_suite(filename)
        if suite is None:
            logger.warning(f'Failed to delete test case {filename}>{test_method_name}')
        else:
            logger.debug(f'Deleting test case {filename}>{test_method_name}')
            suite.remove_testcase(test_method_name)



"""Helper package to facilitate reporting for webdriver-based tests on Tauk"""
import logging
import os
from logging.handlers import RotatingFileHandler
from pathlib import Path

__project__ = "tauk"
__version__ = "2.0.4"
__author__ = "Nathan Krishnan"
__url__ = "https://github.com/thetauk/tauk-webdriver-python"
__platforms__ = "ALL"
__classifiers__ = [
    "Programming Language :: Python :: 3",
    "License :: OSI Approved :: MIT License",
    "Operating System :: OS Independent",
]
__requires__ = ["requests", "filelock", "jsonpickle", "tzlocal"]

__extra_requires__ = {
}


def _init_logger():
    log_filename = os.path.join(Path.home(), '.tauk', 'logs', 'tauk-webdriver.log')
    os.makedirs(os.path.dirname(log_filename), exist_ok=True)
    os.environ['TAUK_HOME'] = os.path.join(Path.home(), '.tauk')

    tauk_logger = logging.getLogger('tauk')
    log_level = os.getenv('TAUK_LOG_LEVEL', 'INFO')
    if log_level.upper() not in ['DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL']:
        log_level = 'INFO'
    tauk_logger.setLevel(logging.getLevelName(log_level.upper()))
    formatter = logging.Formatter(fmt='%(asctime)s [%(process)d-%(threadName)s] %(levelname)s %(message)s',
                                  datefmt='%Y-%m-%dT%H:%M:%S%z')

    file_handler = RotatingFileHandler(log_filename, maxBytes=10000000, backupCount=3)
    file_handler.setFormatter(formatter)
    tauk_logger.addHandler(file_handler)

    stream_handler = logging.StreamHandler()
    # stream_handler.setLevel(logging.ERROR)
    stream_handler.setFormatter(formatter)
    tauk_logger.addHandler(stream_handler)


_init_logger()



import hashlib
import logging
import os
import uuid
import jsonpickle

from tauk.api import TaukApi
from tauk.context.test_data import TestData
from tauk.exceptions import TaukException
from filelock import FileLock

logger = logging.getLogger('tauk')


class TaukContext:

    def __init__(self, api_token, project_id, multi_process_run=False):
        self.test_data: TestData = TestData()
        self.exec_dir = self._get_exec_dir(multi_process_run)
        self.exec_file = os.path.join(self.exec_dir, 'exec.run')
        self.api = TaukApi(api_token, project_id, multi_process_run)

        if multi_process_run:
            self._setup_execution_file()
            return
        else:
            logger.debug('Deleting execution context because its not a multiprocess run')
            self.delete_execution_files()

        self.run_id = self._init_run()

    def _get_exec_dir(self, multi_process_run=False):
        if 'TAUK_EXEC_DIR' in os.environ and len(os.environ.get('TAUK_EXEC_DIR')) > 0:
            logger.debug(f'Using execution dir found in environment variable {os.environ.get("TAUK_EXEC_DIR")}')
            return os.environ.get('TAUK_EXEC_DIR')

        exec_home = os.path.join(os.environ.get('TAUK_HOME'), hashlib.md5(os.getcwd().encode()).hexdigest())

        if not multi_process_run:
            exec_dir = os.path.join(exec_home, 'exec')
            logger.debug(f'Using execution dir at {exec_dir}')
            return exec_dir

        parent_exec_dir = os.path.join(exec_home, f'{os.getppid()}')
        if os.path.exists(parent_exec_dir):
            logger.debug(f'Found exits execution dir at {parent_exec_dir}')
            return parent_exec_dir

        new_exec_dir = os.path.join(exec_home, f'{os.getpid()}')
        logger.debug(f'Using new execution dir at {new_exec_dir}')
        # Set environment variable so that child process can use this
        os.environ['TAUK_EXEC_DIR'] = new_exec_dir
        return new_exec_dir

    def _init_run(self, run_id=None):
        # return self.api.initialize_run_mock(self.test_data, run_id)
        return self.api.initialize_run(self.test_data, run_id)

    def delete_execution_files(self):
        if not os.path.exists(self.exec_dir):
            return

        logger.debug(f'Deleting execution files in {self.exec_dir}')
        lock_file = f'{self.exec_file}.lock'
        if os.path.exists(self.exec_file):
            os.remove(self.exec_file)
        if os.path.exists(lock_file):
            os.remove(lock_file)
        os.rmdir(self.exec_dir)

    def _setup_execution_file(self):
        def set_exec_file(r_id, a_token, p_id):
            with open(self.exec_file, "w") as f:
                logger.debug(f'Updating execution file with {r_id}')
                f.write(f'{r_id},{a_token},{p_id}')
                self.run_id = r_id

        def is_valid_uuid(value):
            try:
                uuid.UUID(value)
                return True
            except ValueError:
                return False

        if not os.path.exists(self.exec_dir):
            os.makedirs(self.exec_dir)

        with FileLock(f'{self.exec_file}.lock', timeout=30):
            logger.debug(f'Execution locked for {self.exec_file}')
            if os.path.exists(self.exec_file):
                with open(self.exec_file, 'r') as file:
                    run_id, api_token, project_id = file.read().strip().split(',')
                    self.api.set_token(api_token, project_id)
                    if is_valid_uuid(run_id):
                        new_run_id = self._init_run(run_id)
                        if new_run_id != run_id:
                            logger.debug(f'Existing runID [{run_id}] is invalid')
                            set_exec_file(new_run_id, api_token, project_id)
                        return
                    else:
                        logger.warning(f'Invalid runID [{run_id}]')

            set_exec_file(self._init_run(), self.api.get_api_token(), self.api.get_project_id())
            logger.debug(f'Execution unlocked for {self.exec_file}')

    def get_json_test_data(self, test_suite_filename, test_method_name):
        suite = self.test_data.get_test_suite(test_suite_filename)
        if not suite:
            raise TaukException(f'Could not find suite with filename {test_suite_filename}')

        suite_json = suite.to_json()

        # Clean up tests
        for test in suite_json.get('test_cases'):
            if test.get('method_name') != test_method_name:
                suite_json.get('test_cases').remove(test)

        json_data = {
            "test_suites": [
                suite_json
            ]
        }

        return jsonpickle.encode(json_data, unpicklable=False)



import logging

from appium.webdriver.common.mobileby import MobileBy
from selenium import webdriver as selenium_driver
from appium import webdriver as appium_driver
from tauk.tauk_webdriver import Tauk

logger = logging.getLogger('tauk')


class CustomIntegrationTest:
    @Tauk.observe(custom_test_name="Test Selenium")
    def test_selenium(self):
        self.driver = selenium_driver.Chrome()
        # self.driver = selenium_driver.Edge()
        Tauk.register_driver(self.driver)
        # CustomIntegrationSupport().another_register_driver(self.driver)
        print('Custom integration test')
        self.driver.get('https://www.tauk.com')
        # assert False, 'Failed with assert'
        # self.driver.quit()

    @Tauk.observe()
    def test_appium(self):
        logger.info('Starting test')
        caps = {
            "deviceName": "Pixel 5",
            "udid": "emulator-5554",
            "platformName": "Android",
            # "browserName": "chrome",
            "appPackage": "io.aj.sample",
            "appActivity": ".MainActivity",
            "automationName": "UiAutomator2",
            "newCommandTimeout": 0
        }
        self.driver = appium_driver.Remote("http://localhost:4723/wd/hub", caps)
        Tauk.register_driver(driver=self.driver)
        self.driver.implicitly_wait(30)
        self.driver.find_element(MobileBy.ID, 'stocksButton').click()
        logger.info('Test complete')


if __name__ == '__main__':
    # Tauk(api_token='test', project_id='test')
    t = CustomIntegrationTest()
    # t.test_selenium()
    t.test_appium()
    t.driver.quit()

    # my_function(2, 7)



import time
import unittest

from tauk.exceptions import TaukException
from tests.utils import mock_success


class UnitTestListenerTestTwo(unittest.TestCase):
    @mock_success(multiprocess=True)
    def test_two(self):
        print('running test two')
        self.assertEqual(True, True, 'failed to assert')  # add assertion here
        # raise TaukException('simple test error')



import argparse
import importlib
import unittest
from unittest import TestLoader

from tauk.listeners.unittest_multiprocess_listener import TaukMultiprocessListener

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('-t', '--test-name', type=str)
    parser.add_argument('-c', '--class-name', type=str)
    args = parser.parse_args()

    module = importlib.import_module(f'tests.e2e.listeners.multiprocess.{args.test_name}')
    test_class = getattr(module, args.class_name)
    suite = unittest.TestSuite()
    for test_method in TestLoader().getTestCaseNames(test_class):
        test_instance = test_class(test_method)
        suite.addTest(test_instance)

    unittest.TextTestRunner(resultclass=TaukMultiprocessListener).run(suite)



import logging

from tauk.listeners.unittest_listener import TaukListener
from tauk.tauk_webdriver import Tauk

logger = logging.getLogger('tauk')


class TaukMultiprocessListener(TaukListener):

    def __init__(self, stream, descriptions, verbosity) -> None:
        logger.debug('Initializing Tauk Multiprocess listener')
        self.multiprocess_run = True
        self.cleanup_exec_context_on_exit = False
        Tauk(multi_process_run=self.multiprocess_run, cleanup_exec_context=self.cleanup_exec_context_on_exit)
        super().__init__(stream, descriptions, verbosity)



import typing

from tauk.context.test_case import TestCase
from tauk.exceptions import TaukException


class TestSuite:
    def __init__(self, filename) -> None:
        self.filename = filename
        self.name = None
        self.class_name = None
        self._test_cases: typing.List[TestCase] = []

    def to_json(self):
        json = {
            'filename': self.filename,
            'name': self.name,
            'class_name': self.class_name,
            'test_cases': [test.to_json() for test in self.test_cases]
        }
        return {k: v for k, v in json.items() if v}

    @property
    def filename(self):
        return self._filename

    @filename.setter
    def filename(self, filename):
        self._filename = filename

    @property
    def name(self) -> str:
        return self._name

    @name.setter
    def name(self, name):
        self._name = name

    @property
    def class_name(self) -> str:
        return self._class_name

    @class_name.setter
    def class_name(self, class_name):
        self._class_name = class_name

    @property
    def test_cases(self):
        return self._test_cases

    def add_testcase(self, testcase: TestCase):
        for test in self.test_cases:
            if test.method_name == testcase.method_name:
                raise TaukException('cannot use TaukListener and Observer() for the same test')

        self.test_cases.append(testcase)

    def remove_testcase(self, test_method_name):
        # TODO: Implement thread safe delete
        pass


    def get_test_case(self, test_name) -> TestCase:
        for test in self.test_cases:
            if test.custom_name == test_name or test.method_name == test_name:
                return test
        return None



import logging
import os
import platform
from datetime import datetime, timezone

import requests

import tauk
from tauk.context.test_data import TestData
from tauk.exceptions import TaukException
from tauk.utils import shortened_json

logger = logging.getLogger('tauk')


class TaukApi:
    run_id: str = None

    def __init__(self, api_token, project_id, multi_process_run=False):
        self._API_URL = os.environ.get('TAUK_API_URL', 'https://www.tauk.com/api/v1')
        self._api_token = api_token
        self._project_id = project_id
        self._multi_process_run = multi_process_run

    def set_token(self, api_token, project_id):
        self._api_token = api_token
        self._project_id = project_id

    def get_api_token(self):
        return self._api_token

    def get_project_id(self):
        return self._project_id

    def initialize_run_mock(self, test_data, run_id=None):
        self.run_id = '5d917db6-cf5d-4f30-8303-6eefc35e7558'
        return self.run_id

    def initialize_run(self, test_data: TestData, run_id: str = None):
        url = f'{self._API_URL}/execution/{self._project_id}/initialize'
        body = {
            'language': test_data.language,
            'tauk_client_version': test_data.tauk_client_version,
            'start_timestamp': int(datetime.now(tz=timezone.utc).timestamp() * 1000),
            'timezone': test_data.timezone,
            'dst': test_data.dst,
            'multi_process_run': self._multi_process_run,
            'host_os_name': platform.system(),
            'host_os_version': platform.platform(terse=True)
        }

        if run_id:
            body['run_id'] = run_id

        headers = {
            'Authorization': f'Bearer {self._api_token}'
        }

        logger.debug(f'Initializing run with: url[{url}], headers[{headers}], body[{body}]')
        response = requests.post(url, json=body, headers=headers)
        if response.status_code != 200:
            logger.error(f'Failed to initialize Tauk execution. Response: {response.text}')
            raise TaukException('failed to initialize tauk execution')

        logger.debug(f'Response: {response.text}')
        self.run_id = response.json()['run_id']
        latest_client_versions = response.json().get('latest_tauk_client_version', tauk.__version__)
        if tauk.__version__ != 'develop' and latest_client_versions != tauk.__version__:
            logger.warning(f'You are currently using Tauk [{tauk.__version__}]. '
                           f'Consider updating to latest version [{latest_client_versions}] using '
                           f'"pip install -U tauk"')
        logger.info(f'Setting run ID for current execution as {self.run_id}')
        return self.run_id

    def test_start(self, test_name, file_name, start_time):
        url = f'{self._API_URL}/execution/{self._project_id}/{self.run_id}/report/test/start'
        body = {
            'test_name': test_name,
            'file_name': file_name,
            'start_time': start_time,
        }

        headers = {
            'Authorization': f'Bearer {self._api_token}'
        }

        response = requests.post(url, json=body, headers=headers)
        logger.info(f'Response: {response.json()}')
        if response.status_code != 200:
            logger.error(f'Failed to upload test. Response: {response.text}')
            raise TaukException('failed to upload test results')

        return response.json()['test_id']

    def test_finish(self, test_name, file_name, start_time, end_time):
        url = f'{self._API_URL}/execution/{self._project_id}/{self.run_id}/report/test/finish'
        body = {
            'test_name': test_name,
            'file_name': file_name,
            'start_time': start_time,
            'end_time': end_time,
        }

        headers = {
            'Authorization': f'Bearer {self._api_token}'
        }

        response = requests.post(url, json=body, headers=headers)
        logger.info(f'Response: {response.json()}')
        if response.status_code != 200:
            logger.error(f'Failed to upload test. Response: {response.text}')
            raise TaukException('failed to upload test results')

        return response.json()['test_id']

    def upload(self, test_data):
        url = f'{self._API_URL}/execution/{self._project_id}/{self.run_id}/report/upload'
        body = test_data

        headers = {
            'Authorization': f'Bearer {self._api_token}',
            'Content-Type': 'application/json'
        }

        logger.debug(f'Uploading test: url[{url}], headers[{headers}], body[{shortened_json(body)}]')
        response = requests.post(url, data=body, headers=headers)
        if response.status_code != 200:
            logger.error(f'Failed to upload test. Response: {response.text}')
            raise TaukException('failed to upload test results')

        logger.debug(f'Response: {response.text}')



import logging
from enum import Enum, unique

from tauk.exceptions import TaukException

logger = logging.getLogger('tauk')


@unique
class TestStatus(Enum):
    FAILED = 'failed'
    PASSED = 'passed'
    EXCLUDED = 'excluded'

    def __getstate__(self):
        return self.value


@unique
class BrowserNames(Enum):
    CHROME = 'chrome'
    FIREFOX = 'firefox'
    EDGE = 'edge'
    SAFARI = 'safari'

    def __getstate__(self):
        return self.value

    @classmethod
    def resolve(cls, name: str):
        if 'chrome' in name.lower():
            return BrowserNames.CHROME
        elif 'firefox' in name.lower():
            return BrowserNames.FIREFOX
        elif 'edge' in name.lower():
            return BrowserNames.EDGE
        elif 'safari' in name.lower():
            return BrowserNames.SAFARI
        else:
            logger.warning(f'Unsupported browser name {name}')


@unique
class AutomationTypes(Enum):
    APPIUM = 'appium'
    SELENIUM = 'selenium'
    ESPRESSO = 'espresso'
    XCTEST = 'xctest'

    def __getstate__(self):
        return self.value


@unique
class PlatformNames(Enum):
    IOS = 'ios'
    ANDROID = 'android'
    WINDOWS = 'windows'
    LINUX = 'linux'
    MACOS = 'macos'

    def __getstate__(self):
        return self.value

    @classmethod
    def resolve(cls, name: str):
        if 'mac' in name.lower():
            return PlatformNames.MACOS
        elif 'windows' in name.lower():
            return PlatformNames.WINDOWS
        elif 'linux' in name.lower():
            return PlatformNames.LINUX
        else:
            raise TaukException(f'unable to resolve platform name {name}')



import logging
import re
from contextlib import suppress

import tzlocal
import inspect

import traceback
import typing

from tauk.context.test_error import TestError
from tauk.enums import AutomationTypes, PlatformNames, TestStatus, BrowserNames
from tauk.exceptions import TaukException
from tauk.utils import get_appium_server_version, get_browser_driver_version

logger = logging.getLogger('tauk')


class TestCase(object):

    def __init__(self) -> None:
        self._id: str = None
        self._custom_name: str = None
        self._method_name: str = None
        self._status: TestStatus = None
        self.excluded: bool = False
        self._automation_type: AutomationTypes = None
        self._platform_name: PlatformNames = None
        self.platform_version: str = None
        self.browser_name: str = None
        self.browser_version: str = None
        self._start_timestamp: int = None
        self._end_timestamp: int = None
        self.timezone: str = None  # "America/Los_Angeles"
        self._error: TestError = None
        self.screenshot: str = None
        self.view: str = None
        self._code_context: typing.List[object] = None
        self.webdriver_client_version: str = None
        self.browser_driver_version: str = None
        self.appium_server_version: str = None
        self._capabilities: {} = None
        self._tags: {} = None
        self._user_data: {} = None
        self.log: typing.List[object] = None

        self._driver_instance = None

    # NOTE: Any object that should be a part of test case should be explicitly added to to_json()
    #       method
    def to_json(self):
        json = {
            'id': self.id,
            'custom_name': self.custom_name,
            'method_name': self.method_name,
            'status': TestStatus.EXCLUDED if self.excluded else self.status,
            'automation_type': self.automation_type,
            'platform_name': self.platform_name,
            'platform_version': self.platform_version,
            'browser_name': self.browser_name,
            'browser_version': self.browser_version,
            'start_timestamp': self.start_timestamp,
            'end_timestamp': self.end_timestamp,
            'timezone': self.timezone,
            'error': self.error,
            'screenshot': self.screenshot,
            'view': self.view,
            'code_context': self.code_context,
            'webdriver_client_version': self.webdriver_client_version,
            'browser_driver_version': self.browser_driver_version,
            'appium_server_version': self.appium_server_version,
            'capabilities': self.capabilities,
            'tags': self.tags,
            'user_data': self.user_data,
            'log': self.log,
        }

        return {k: v for k, v in json.items() if v}

    @property
    def id(self):
        return self._id

    @id.setter
    def id(self, id):
        self._id = id

    @property
    def custom_name(self):
        return self._custom_name

    @custom_name.setter
    def custom_name(self, custom_name):
        self._custom_name = custom_name

    @property
    def method_name(self):
        return self._method_name

    @method_name.setter
    def method_name(self, method_name):
        self._method_name = method_name

    @property
    def status(self):
        return self._status

    @status.setter
    def status(self, status: TestStatus):
        if not isinstance(status, TestStatus):
            raise TaukException(f'status [{status}] must be of type TestStatus')
        self._status = status

    @property
    def automation_type(self):
        return self._automation_type

    @automation_type.setter
    def automation_type(self, automation_type: AutomationTypes):
        self._automation_type = automation_type

    @property
    def platform_name(self):
        return self._platform_name

    @platform_name.setter
    def platform_name(self, platform_name: PlatformNames):
        self._platform_name = platform_name

    @property
    def start_timestamp(self):
        return self._start_timestamp

    @start_timestamp.setter
    def start_timestamp(self, start_timestamp):
        self.timezone = tzlocal.get_localzone_name()
        self._start_timestamp = start_timestamp

    @property
    def end_timestamp(self):
        return self._end_timestamp

    @end_timestamp.setter
    def end_timestamp(self, end_timestamp):
        self._end_timestamp = end_timestamp

    @property
    def error(self):
        return self._error

    @error.setter
    def error(self, error):
        self._error = error

    @property
    def code_context(self):
        return self._code_context

    @code_context.setter
    def code_context(self, code_context):
        self._code_context = code_context

    @property
    def capabilities(self):
        return self._capabilities

    @capabilities.setter
    def capabilities(self, capabilities):
        self._capabilities = capabilities

    @property
    def tags(self):
        return self._tags

    def add_tag(self, tag_name, value):
        logger.debug(f'Adding tag {tag_name}={value}')
        self._tags[tag_name] = value

    @property
    def user_data(self):
        return self._user_data

    def add_user_data(self, name, value):
        if len(name) > 100 and len(value) > 1000:
            raise TaukException('user data is too large')
        self._user_data[name] = value

    @property
    def driver_instance(self):
        return self._driver_instance

    @driver_instance.setter
    def driver_instance(self, driver_instance):
        self._driver_instance = driver_instance

    def register_driver(self, driver, test_filename=None, test_method_name=None):
        if not driver or 'webdriver' not in f'{type(driver)}':
            raise TaukException(f'Driver {type(driver)} is not of type webdriver')

        if test_filename:
            driver.tauk_test_filename = test_filename
        if test_method_name:
            driver.tauk_test_method_name = test_method_name

        def tauk_callback(func):
            def inner():
                self.capture_screenshot()
                self.capture_view_hierarchy()
                func()

            return inner

        driver.quit = tauk_callback(driver.quit)

        self.driver_instance = driver
        self.capabilities = driver.capabilities

        # Collect other data about driver
        if 'selenium' in f'{type(driver)}':
            self.automation_type = AutomationTypes.SELENIUM
            with suppress(Exception):
                import selenium as s
                self.webdriver_client_version = s.__version__
        elif 'appium' in f'{type(driver)}':
            self.automation_type = AutomationTypes.APPIUM
            with suppress(Exception):
                from appium.common.helper import library_version
                self.webdriver_client_version = library_version()
                self.appium_server_version = get_appium_server_version(driver)

        if self.automation_type is AutomationTypes.APPIUM:
            self.platform_name = PlatformNames.resolve(self.capabilities.get('platform', ''))
        else:
            self.platform_name = PlatformNames.resolve(self.capabilities.get('platformName', ''))

        self.platform_version = self.capabilities.get('platformVersion', None)

        if self.capabilities.get('browserName', None):
            self.browser_name = BrowserNames.resolve(self.capabilities.get('browserName', ''))
            self.browser_version = self.capabilities.get('browserVersion', None)
            self.browser_driver_version = get_browser_driver_version(driver)

    def capture_screenshot(self):
        if self.screenshot and len(self.screenshot) > 0:
            logger.debug('Screenshot is already captured')
            return
        if self.driver_instance:
            try:
                self.screenshot = self.driver_instance.get_screenshot_as_base64()
            except Exception as ex:
                logger.error("An issue occurred while trying to take a screenshot.", exc_info=ex)

    def capture_view_hierarchy(self):
        if self.view and len(self.view) > 0:
            logger.debug('View Hierarchy is already captured')
            return
        if self.driver_instance:
            try:
                if hasattr(self.driver_instance, 'contexts') and 'FLUTTER' in self.driver_instance.contexts:
                    current_context = self.driver_instance.current_context
                    self.driver_instance.switch_to.context('NATIVE_APP')
                    self.view = self.driver_instance.page_source
                    self.driver_instance.switch_to.context(current_context)
                    return

                self.view = self.driver_instance.page_source
            except Exception as ex:
                logger.error("An issue occurred while capturing view hierarchy.", exc_info=ex)

    def capture_error(self, caller_filename, exec_info):
        exc_type, exc_value, exc_traceback = exec_info
        stack_summary_list = traceback.extract_tb(exc_traceback)
        filename, line_number, invoked_func, code_executed = None, None, None, None
        for stack_trace in stack_summary_list:
            if caller_filename in stack_trace.filename:
                filename, line_number, invoked_func, code_executed = stack_trace
                break

        self.error = {
            'error_type': exc_value.__class__.__name__,
            'error_msg': str(exc_value),
            'line_number': line_number,
            'invoked_func': invoked_func,
            'code_executed': code_executed,
            'traceback': traceback.format_exc()
        }
        return line_number

    def capture_success_data(self):
        self.status = TestStatus.PASSED
        self.capture_screenshot()
        self.capture_view_hierarchy()

    def capture_failure_data(self):
        self.status = TestStatus.FAILED
        self.capture_screenshot()
        self.capture_view_hierarchy()

    def capture_test_steps(self, testcase, error_line_number=0):
        testcase_source_raw = inspect.getsourcelines(testcase)
        testcase_source_clean = [step.strip() for step in testcase_source_raw[0]]
        line_number = testcase_source_raw[1]

        # Discard the initial 2 lines as they are the decorator and the testcase function name
        testcase_source_clean = testcase_source_clean[2:]
        line_number += 2

        output = []
        for step in testcase_source_clean:
            output.append(
                {
                    'line_number': line_number,
                    'line_code': step
                }
            )
            line_number += 1

        if error_line_number > 0:
            for index, value in enumerate(output):
                if value['line_number'] == error_line_number:
                    # get previous 9 lines plus the line where the error occurred
                    # ensure that the start range value is never below zero
                    # get the next 9 lines after the error occured
                    # ensure that the end range value never exceeds the len of the list
                    result = output[max(index - 9, 0): min(index + 10, len(output))]
                    self.code_context = result
        else:
            self.code_context = output

    def capture_appium_logs(self):
        def format_appium_log(log_list):
            output = []
            for event in log_list:
                # ANSI escape sequences
                # https://bit.ly/3rK88pe
                ansi_escape = re.compile(r'\x1B(?:[@-Z\\-_]|\[[0-?]*[ -/]*[@-~])')

                # Split at first event type occurence
                # in square brackets, e.g. [HTTP]
                # https://bit.ly/3fItHEi
                split_log_msg = re.split(r'\[|\]', ansi_escape.sub(
                    '', event['message']), maxsplit=2)

                formatted_event = {
                    'timestamp': event.get('timestamp'),
                    'level': event.get('level'),
                    'type': split_log_msg[1],
                    'message': split_log_msg[2].strip()
                }

                output.append(formatted_event)
            return output

        if self.automation_type != AutomationTypes.APPIUM:
            return

        # Get last 50 log entries
        # minus the 5 log entries for issuing get_log()
        slice_range = slice(-55, -5)

        if not self.driver_instance:
            logger.warning('Not capturing appium logs because driver instance was not registered')
            return

        try:
            self.log = format_appium_log(self.driver_instance.get_log('server')[slice_range])
        except Exception as ex:
            logger.error('An issue occurred while requesting the Appium server logs.', exc_info=ex)
